/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * COPYING NOTES:
 *
 * iofile.h -- Advanced I/O file managing functions header
 *
 * Copyright (C) 2000 Roberto A. Foglietta <robang@libero.it>
 * Copyright (C) 2002 GEA-Automotive <fogliettar@gea-automotive.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * REVISION NOTES:
 *
 * released 16-10-2002 by Roberto A. Foglietta
 * enlarged 30-05-2003 by Roberto A. Foglietta
 * c++ready 09-07-2003 by Gianfranco Pesce
 * enlarged 14-07-2005 by Roberto A. Foglietta
 */
 

#ifdef __cplusplus
extern "C" {
#endif //__cplusplus

#ifndef MAX_STDIN_BLOCK
#define MAX_STDIN_BLOCK 10240	//default value
#endif  

#ifndef _LIBIOFILE_H
#define _LIBIOFILE_H

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Include that are necessary or supposed usefull
 */
#include <stdio.h>   //declaration of FILE* type variable for the file pointer
#include <stdlib.h>  //useful for function 'free' to dealloc the file buffer
 

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Function syntax  
 */ 


char *basename (const char *name);
/*
 * base = basename (const char *name);
 *
 * this function return a pointer of a allocated string containing the basename
 * or NULL in case of error.
 */


FILE *check_n_openfile (const char *name, const char *mode);
/*
 * filep = check_n_openfile ( name, mode );
 *
 * this function return the pointer 'filep' to the file which 'name' was passed
 * as first argument opened in 'mode' (es.: "r+", "wt", etc.). In case of fatal 
 * error 'filep' will be set to NULL.
 */


unsigned long get_file_lenght (FILE * fp);
/*
 * lung = get_file_lenght ( filep );
 *
 * this function return the lenght 'lung' of the file pointed by 'filep' passed
 * as first parameter, that point to opened file which you're interested in.
 */


#define read_file_to_buffer read_from_file
/*
 * buffer = read_file_to_buffer (name, mode, &lun);
 *
 * for backcompatibility with old iofile libraries
 */ 


unsigned char *read_from_file (const char *name, const char *mode, long *lun);
/*
 * buffer = read_from_file (name, mode, &lun);
 *
 * this function try to access to the file specified by path in the first 
 * parameter 'name' and check if it possibile open it in 'mode' (es.: "r+",
 * "rt", etc.), check the lenght and store it in 'lun', alloc the necessary 
 * memory, open the file and read it into allocated buffer. If each of these
 * will succeded it will return the pointer to allocated buffer which contain
 * the file contenent otherwise it will return NULL.
 */

unsigned char *read_from_stdin(unsigned long *len);
/*
 * buffer = read_from_stdin (&lun);
 *
 * this function read from the input until EOF (good for piping) and realloc 
 * memory as far as it needs. Returns the pointer to the allocated buffer or
 * NULL in case of error. In the variable lun will be found the number of read 
 * bytes even in case of error so buffer != NULL has to be checked.
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Enlarged function syntax  
 */ 

int  create_pid_file(const char *name, int signum);
int  remove_pid_file(const char *name);
long readto_pid_file(const char *name);

#endif //_LIBIOFILE_H

#ifdef __cplusplus
}
#endif //__cplusplus
