/* 
 * (C) 2005, Roberto A. Foglietta
 *     Under public domain licence
 */
 
 

#ifndef DEBUG_H
# define DEBUG_H

# include <syslog.h>
# include <stdarg.h>

# define INITLOG(name, log, lev)	do { 					\
	openlog(name, LOG_NDELAY|LOG_PERROR, (log));	\
	setlogmask(LOG_UPTO(lev));						\
} while(0)

# define PRINTF(x...)	do { 							\
	if(_debug) {										\
		printf("DBG %s (%d)> ", __FILE__, __LINE__); 	\
		printf(x); 										\
		printf("\n"); 									\
	}													\
} while(0)

# define LOGGATE(lev, x...)   do { 								\
	if(_debug)					 								\
		fprintf(stderr, "DBG %s (%d)> ", __FILE__, __LINE__);	\
	syslog((_debug)?7:(lev), x);								\
	if(_debug) 													\
		fprintf(stderr,"\n"); 									\
} while(0)															

# define LOGDBUG(x...)	LOGGATE(LOG_DEBUG, x)
# define LOGINFO(x...)	LOGGATE(LOG_INFO, x)
# define LOGWARN(x...)	LOGGATE(LOG_WARNING, x)
# define LOGERRO(x...)	LOGGATE(LOG_ERR, x)
# define DEBUG(a)  static int _debug = (a)

//# define LOGLEV(a) static int _loglev = (a)

# define ON  1
# define OFF 0
#endif /* DEBUG_H */

/*
#include <sys/time.h>
static inline long myclock()
{
 struct timeval tv;

 gettimeofday (&tv, NULL);
 return (tv.tv_sec * 1000 + tv.tv_usec / 1000);
}
*/
