/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * COPYING NOTES
 *
 * iofile.c -- Advanced I/O file managing functions
 *
 * Copyright (C) 2000 Roberto A. Foglietta <robang@libero.it>
 * Copyright (C) 2002 GEA-Automotive <fogliettar@gea-automotive.com>
 * Copyright (C) 2004 Roberto A. Foglietta <me@roberto.foglietta.name>
 * Copyright (C) 2005 Roberto A. Foglietta <roberto.foglietta@sad.it>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * REVISION NOTES:
 *
 * released 16-10-2002 by Roberto A. Foglietta
 * modified 24-12-2002 by Roberto A. Foglietta
 * bugfixed 04-01-2005 by Roberto A. Foglietta
 * features 16-06-2005 by Roberto A. Foglietta
 * function 14-07-2005 by Roberto A. Foglietta
 *
 */


#include <stdlib.h>
#include <unistd.h>
#include <memory.h>
#include "iofile.h"  
#include "rafdebug.h"
DEBUG(0);

/***********************************************************************

 	FUNZIONI DI SUPPORTO

***********************************************************************/

char *
basename (const char *name) {
	const char *p = NULL;

	if(!name)
		return NULL;

	if(!(p = strrchr(name, '/')))
		p = name;
	else	
		p++;
	return strdup(p);
}


FILE *
check_n_openfile (const char *name, const char *mode)
{
	int flag, i, len;
	flag = R_OK | F_OK;
	len = strlen (mode);
	for (i = 0; i < len; i++) {
		if (mode[i] == 'w' || mode[i] == 'a' || mode[i] == '+') {
			flag |= W_OK;
			break;
		}
	}
	if (access (name, flag))
		return NULL;
	else
		return fopen (name, mode);
}


unsigned long
get_file_lenght (FILE * fp)
{
	long lun;
	
	fflush (fp);
	fseek (fp, 0, SEEK_END);
	fflush (fp);
	lun = ftell (fp);
	rewind (fp);
	fflush (fp);
	
	return lun;
}


unsigned char *
read_from_file (const char *name, const char *mode, long *lun)
{
	FILE *fp;
	unsigned long len;
	unsigned char *buf;

	if(lun)
		*lun = 0;

	fp = check_n_openfile (name, mode);
	if (fp == NULL)
		return NULL;
	else
		PRINTF ("Aperto il file\n");
	
	len = get_file_lenght (fp);
	if (len <= 0)
		return NULL;
	else
		PRINTF ("lunghezza %ld\n", len);
	
	buf = (unsigned char *) malloc (len+16);
	if (buf == NULL)
		return NULL;
	else
		PRINTF ("Allocata la memoria\n");
	
	len = (long) read (fileno (fp), buf, (size_t) len);
	PRINTF ("letti %ld bytes\n", len);
	fclose (fp);
	
	if (len <= 0) {
		free (buf);
		buf = NULL;
	} else {
		PRINTF ("Lettura file OK: 0x%lx\n", (unsigned long) buf);
		if(lun)
			*lun = len;
		buf[len] = 0;
	}
	
	return buf;
}



unsigned char *
read_from_stdin(unsigned long *len) 
{
	int c;
	unsigned long i, k = 0; 
	unsigned char *buf = NULL;

	i = 0;
	while((c = getchar()) != EOF) {
		if(i+1 > k) {
			k += MAX_STDIN_BLOCK;
			buf = (unsigned char *)realloc(buf, k);
			if(buf == NULL) {
				perror("ERROR on realloc()");
				return NULL;
			}
		}
		buf[i++] = c;
		*len = i; 
	} 

	return buf;
}
