/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * COPYING NOTES:
 *
 * pidfile.c -- lock pid file managing little library
 *
 * Copyright (C) 2000 Roberto A. Foglietta <robang@libero.it>
 * Copyright (C) 2002 GEA-Automotive <fogliettar@gea-automotive.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

#include "iofile.h"
#include <unistd.h>
#include <sys/types.h>
#include <signal.h>
//#include <stdio.h>


long
readto_pid_file(const char *name)
{
	long pid;
	unsigned long lun;
	unsigned char * buf = read_file_to_buffer (name, "r", &lun);
	
	if(buf == NULL) {
		fprintf(stderr,"\nI cannot read file %s\n", name);	
		return -1;
	} 
	
	pid = atol(buf);
	if(pid <= 0) 
		return -1;
	else
		return pid;
}


int
create_pid_file(const char *name, int signal)
{
	FILE *fp;
	int ret = 0;
	long pid = readto_pid_file(name);

	if(pid >= 1 && kill(pid,signal)) {
		perror("\ncreate_pid_file> kill"); 
		fprintf(stderr,"ERROR: I cannot kill process %ld with signal %d\n", pid, signal);
		ret = 1;
	}

	fp = fopen(name, "w");
	if(fp == NULL) {
		fprintf(stderr,"\nFATAL ERROR: I cannot open file %s\n", name);
		return -1;
	}

	pid = (long)getpid();
	fprintf(fp, "%ld\n", pid);
	fclose(fp);

	return ret;
}


int
remove_pid_file(const char *name)
{
	long pid;
	unsigned long lun;
	unsigned char * buf = read_file_to_buffer (name, "r", &lun);

	if(buf) {
		pid = atol(buf);
		if(pid >= 1 && pid != (unsigned long)getpid()) {
			fprintf(stderr,"\nERROR: I cannot remove %s because it's belong to sombody else (%ld)!\n", name, pid);
			return -1;
		} else if(remove(name)) {
			perror("remove_pid_file> remove");
			return -2;
		}
	} //else fprintf(stderr,"\nATTENTION: lock file %s doesn't exist!\n", name);
	
	return 0;
}
