/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * COPYING NOTES
 *
 * ServerSocket.cpp -- Implementation of the ServerSocket class
 * 
 * Copyright (C) 2002, Roberto A. Foglietta <robang@libero.it>
 * Copyright (C) 2002, GEA S.r.l. Generale Elettronica Automotive
 *
 * Copyright (C) 2004, Roberto A. Foglietta <me@roberto.foglietta.name>
 * Copyright (C) 2004, Tecno Elettrica Brianza snc di Riva P. & G.  
 *
 * Copyright (C) 2005, Roberto A. Foglietta <me@roberto.foglietta.name>
 * Copyright (C) 2005, SAD Trasporti Locali Bolzano s.p.a.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * REVISION NOTES:
 * released 21-10-2002 by Roberto A. Foglietta
 */


#include "ServerSocket.h"
#include "SocketException.h"


ServerSocket::ServerSocket (int port)
{
	zread = 0;

	if (!Socket::create ()) {
		throw SocketException ("Could not create server socket.");
	}

	if (!Socket::bind (port)) {
		throw SocketException ("Could not bind to port.");
	}

	if (!Socket::listen ()) {
		throw SocketException ("Could not listen to socket.");
	}
	
	Socket::set_non_blocking (true);
}


ServerSocket::~ServerSocket () 
{
}


const ServerSocket &
ServerSocket::operator << (const std::string & s) const
{
	if (!Socket::send (s)) {
		throw SocketException ("Could not write to socket.");
	}

	return *this;
}

//#include <iostream>

const ServerSocket &
ServerSocket::operator >> (std::string & s) //const
{
	int ret = Socket::recv (s);
	if (ret < 0) {
		throw SocketException ("Socket not ready.");
	} else 
	if (ret > 0) {
		zread = 0;
	} else {
		zread++;
		//std::cerr << "zread = " << zread << "\n";
		throw SocketException ("Zero data read from socket (closed?).");
	}

	return *this;
}


void
ServerSocket::accept (ServerSocket & sock)
{
	zread = 0;

	if (!Socket::accept (sock)) {
		throw SocketException ("Could not accept socket.");
	}
	
	Socket::set_non_blocking (true);
}


void
ServerSocket::close (ServerSocket & sock)
//ServerSocket::close ()
{
	zread = 0;

	if (!Socket::close (sock)) {
		throw SocketException ("Could not close socket.");
	}
}
