/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * COPYING NOTES
 *
 * Socket.h -- Definition of the Socket class
 *
 * Copyright (C) 2002, Roberto A. Foglietta <robang@libero.it>
 * Copyright (C) 2002, GEA S.r.l. Generale Elettronica Automotive
 *
 * Copyright (C) 2004, Roberto A. Foglietta <me@roberto.foglietta.name>
 * Copyright (C) 2004, Tecno Elettrica Brianza snc di Riva P. & G.  
 *
 * Copyright (C) 2005, Roberto A. Foglietta <me@roberto.foglietta.name>
 * Copyright (C) 2005, SAD Trasporti Locali Bolzano s.p.a.
 * 
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * REVISION NOTES:
 * released 21-10-2002 by Roberto A. Foglietta
 * modified 12-04-2005 by Roberto A. Foglietta
 * modified 01-08-2005 by Roberto A. Foglietta
 */
 
#ifndef Socket_class
#define Socket_class

#include <sys/poll.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <unistd.h>

#include <string>

const int MAXRECV = 2048;
const int MAXHOSTNAME = 256;
const int MAXCONNECTIONS = 5;
const int POLL_TIMEOUT_ms = 10;

class Socket
{
 public:
	
	Socket ();
	virtual ~ Socket ();

	// Server initialization
	bool create ();
	bool close (Socket &) const;
	//bool close () const;
	bool bind (const int port);
	bool listen () const;
	bool accept (Socket &) const;

	// Client initialization
	bool connect (const std::string host, const int port);

	// Data Transimission
	bool send (const std::string) const;
	int recv (std::string &) const;

	// Socket proprieties
	bool is_valid () const { return (m_sock >= 0); }
	void set_non_blocking (const bool);
	void set_timeout(const int msec) const { timeout(msec, NULL); }
	int  is_ready() const { return (timeout(-1, NULL) > 0); }
	int  is_hangup() const;

 private:

	int m_sock;
	sockaddr_in m_addr;
	int timeout (const int msec, int *mask) const;

};


#endif
