/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * COPYING NOTES
 *
 * server.cpp -- main server body
 * 
 * Copyright (C) 2002, Roberto A. Foglietta <robang@libero.it>
 * Copyright (C) 2002, GEA S.r.l. Generale Elettronica Automotive
 *
 * Copyright (C) 2004, Roberto A. Foglietta <me@roberto.foglietta.name>
 * Copyright (C) 2004, Tecno Elettrica Brianza snc di Riva P. & G.  
 *
 * Copyright (C) 2005, Roberto A. Foglietta <me@roberto.foglietta.name>
 * Copyright (C) 2005, SAD Trasporti Locali Bolzano s.p.a.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * REVISION NOTES:
 * released 21-10-2002 by Roberto A. Foglietta
 * modified 16-12-2004 by Roberto A. Foglietta
 * modified 12-04-2005 by Roberto A. Foglietta
 * modified 24-06-2005 by Roberto A. Foglietta
 */
 
#include <iostream>
#include <cstring>
#include <string>
#include <cassert>

#include <unistd.h>

#include "ServerSocket.h"
#include "SocketException.h"

#include "mysignal.h"
#include "replyfunc.h"

extern "C"
{
 #include "server.h"
 
 int my_socket_server(const char *host, int port, void *reply, void *alarm, void *mysql)
 {	
	int i, acpt[MAXCONNECTIONS];
	for(i = 0; i < MAXCONNECTIONS; i++)
		acpt[i] = false;
	 
	if(!host) 
		host = _HOST;
	if(port <= 0) 
		port = _PORT;
	
	std::cerr << "Connecting to " << host << " and running on port " << port << "...\n";	
	try {
		// Create the socket
		ServerSocket server (port);
		ServerSocket new_sock[MAXCONNECTIONS];
	
		while(true) {
			if (CTRL_C_FLAG) {
				int j;
				std::cerr << "Sent to all accepted sockets:\n" << _STOP;
				for(j = 0; j < MAXCONNECTIONS; j++)
					if(acpt[j]) {
						new_sock[j] << _STOP;
						acpt[j] = false;
						server.close(new_sock[j]);
						//new_sock[j].close();
					}
				return 0;
			} else
				usleep(POLLING_PAUSE/MAXCONNECTIONS);

			if (alarm != NULL) {
				((ALARM_FNC)alarm)();
			}
		
			i++; 
			if(i >= MAXCONNECTIONS) 
				i = 0;
			
			
			char *str;
			std::string data;
		
			if(!acpt[i]) {
				try {
					server.accept(new_sock[i]);
					std::cerr << "Socket n." << i << " accepted connection\n";
					acpt[i] = true;
				} catch (SocketException & e) { continue; }
			}
		
			if(!acpt[i]) 
				continue;
						
			try {
				//std::cerr << "Reading data from socket n." << i << "...\n";
				new_sock[i] >> data; 
				acpt[i] = 1;	//reset timeout
				std::cerr << "Received data from socket n." << i << ":\n" << data << "\n";
				/************ TEST CODE ******************
				unsigned i; 
				for(i = 0; i < data.size(); i++)
					fprintf(stderr, "0x%02x ", (data.c_str())[i]);
				if(i)
					fprintf(stderr, "\n\n");
				*****************************************/
			} catch (SocketException & e) {
				//std::cerr << "Socket n." << i << " read error: " << e.description () << "\n";
				if(acpt[i]) {
					if(acpt[i]++ > NOUSE_TIMEOUT) {
						acpt[i] = false;
						server.close(new_sock[i]);
						//new_sock[i].close();
						std::cerr << "Socket n." << i << " closed\n";						
					} else 
					if(!new_sock[i].alive()) {
						acpt[i] = false;
						server.close(new_sock[i]);
						//new_sock[i].close();
						std::cerr << "Socket n." << i << " hangup\n";											
					}
				}
				continue;
			}	

			if ((str = ((REPLY_FNC)reply)(data.c_str (), mysql))) {
				int stp = !strncasecmp(str, _STOP, strlen(_STOP));
				new_sock[i] << (std::string) str;
				std::cerr << "Replay sent:\n" << (std::string) str << "\n";
				free(str);
				if(stp)	{
					acpt[i] = false;
					server.close(new_sock[i]);
					//new_sock[i].close();
					std::cerr << "Socket n." << i << " closed\n";
				}
			} else {
				new_sock[i] << _FAIL;
				std::cerr << "sent:\n" << _FAIL;
			}		
		}
	} catch (SocketException & e) {
		std::cerr << "Creating server socket exception was caught:" << e.description () << "\nExiting.\n";
		return 1;
	}

	return 0;
 }
} //externC
