/***************************************************************************
 *            mainargs.c
 *
 *  Wed Apr  6 17:33:03 2005
 *  Copyright  2005  Roberto A. Foglietta
 *  roberto.foglietta@sad.it
 ****************************************************************************/
 
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <getopt.h>

char *username = NULL;
char *password = NULL; 
char *hostname = NULL;
char *dbname   = NULL;
char *prgName  = NULL;
unsigned int dbport = 0;
unsigned int skport = 0;

/*!	\brief Mostra come si passano al programma i parametri a linea di comando e poi esce con \c $?=1
 * 	\param s il nome dell'eseguibile
  */
static
void usage(char *s)
{
#ifdef USE_MYSQL
	fprintf(stderr,"\nUsage: %s [-v|-h] [-u<username>|-p<password>] [-H<hostname>] [-P<dbport>] [-D<dbname>] skport\n\n",s);
#else
	fprintf(stderr,"\nUsage: %s [-v|-h] [-H<hostname>] socketport\n\n",s);
#endif
}


/*!	\brief Il vero nome dell'applicazione
	\param s stringa che contiene il path completo
	\return puntatore al nome del binario che estrapolanto dal path completo
 */
static
char *true_name(char *s)
{
	char	*t = strrchr(s,'/');
	return	(t==NULL) ? s : t+1;
}

static
const char *longargs[]={"help", "version", NULL};
enum { 	
	o_HELP = 0,
	o_VERS,  
	o_NMAX 
};

/*!	\brief Inizializza i valori dei parametri secondo gli argomenti passati da linea di comando
	\param argc il numero dei parametri passategli da linea di comando
	\param argv i parametri passategli da linea di comando
	\param version serve per mostrare la versione dell'eseguibile
	\return \c 0 se tutto va bene
 */
int setup_args(int argc, char **argv, char *version) 
{
	int i, op;
	struct option *opl[o_NMAX];
		
	for (i = 0; i < o_NMAX; i++) {
		opl[i] = (struct option *)malloc(sizeof(struct option));
		opl[i]->name = strdup(longargs[i]);
		opl[i]->has_arg = no_argument;
		opl[i]->flag = NULL;
		opl[i]->val = i;
	}
	opl[i] = NULL;

	opterr  = 0;
	prgName = true_name(argv[0]);
	while ( (op = getopt_long(argc, argv, "vhu:p:H:P:D:", opl[0], NULL)) != EOF ) 
		switch ( op ) {
		case o_VERS:
		case 'v':
			fprintf(stderr, "versione %s\n\n", version);
			return 1;
		case o_HELP:
		case 'h':
			usage(prgName);		
			return 2;
#ifdef USE_MYSQL
		case 'u':
			username = strdup(optarg);
			break;
		case 'p':
			password = strdup(optarg);
			break;
		case 'P':
			dbport =  atoi(optarg);
			break;
		case 'D':
			dbname =  strdup(optarg);
			break;
#endif
		case 'H':
			hostname =  strdup(optarg);
			break;
		default:
			printf("\nop: %d (%c)\n", op, op);
			usage(prgName);
			return -1;
	}
	
	if ( optind<argc )
		skport = atoi(argv[optind++]);

	return 0;
}
