/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * COPYING NOTES
 *
 * replyfunc.c -- replay function bodies
 *
 * Copyright (C) 2002, Roberto A. Foglietta <robang@libero.it>
 * Copyright (C) 2002, GEA S.r.l. Generale Elettronica Automotive
 *
 * Copyright (C) 2004, Roberto A. Foglietta <me@roberto.foglietta.name>
 * Copyright (C) 2004, Tecno Elettrica Brianza snc di Riva P. & G.  
 *
 * Copyright (C) 2005, Roberto A. Foglietta <me@roberto.foglietta.name>
 * Copyright (C) 2005, SAD Trasporti Locali Bolzano s.p.a.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * REVISION NOTES:
 * released 21-10-2002 by Roberto A. Foglietta
 * modified 08-04-2005 by Roberto A. Foglietta
 * modified 12-04-2005 by Roberto A. Foglietta
 * modified 24-06-2005 by Roberto A. Foglietta
 */


#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include "rafdebug.h"
#include "replyfunc.h"


#define MAXLEN 256
#define is_atok_char(c) ((c) == ' ' || (c) == '\n' || (c) == '\r' || (c) == '\0')

#ifdef USE_MYSQL
# include <mysql/mysql.h>
//# include "iofile.h"
 char *		ACT_TEXT[N_ACT_FNC] = { "hello",	"query",   "help",   "bye" };
 REPLY_FNC	ACT_FUNC[N_ACT_FNC] = {  act_hello,	act_query, act_help, act_bye };
#else
 char *		ACT_TEXT[N_ACT_FNC] = { "hello",	"help",   "bye" };
 REPLY_FNC	ACT_FUNC[N_ACT_FNC] = {  act_hello,	act_help, act_bye };
#endif

char *act_hello(const char *data, void *mysql)
{
	char name[MAXLEN];
	int n = sprintf(name, "Hello from ");
	gethostname(&name[n], MAXLEN-n-2);
	strcat(name, "\n");
	
	return strdup(name);
}

char *act_bye(const char *data, void *mysql)
{
	return strdup(_STOP);
}

char *act_help(const char *data, void *mysql)
{
	int i, j = MAXLEN;
	char str[MAXLEN];

	sprintf(str, "Admitted command list:\n");
	for (i = 0; i < N_ACT_FNC && j; i++) {
		strcat(str, ACT_TEXT[i]);
		j -= strlen(ACT_TEXT[i]);
		strcat(str, "\n");
		j --;
	}
	return strdup(str);
}
#undef MAXLEN

char *reply_func (char const *data, void *mysql)
{
	DEBUG(0);
#ifdef USE_MYSQL
	if(dbQueryMode)
		return ((REPLY_FNC)(ACT_FUNC[QUERY]))(data, mysql);
#endif

	int i, len;
	for (i = 0; i < N_ACT_FNC; i++) {
		len = strlen(ACT_TEXT[i]);
		PRINTF("i=%d, len=%d, data='%s' act_text='%s'\n", i, len, data, ACT_TEXT[i]);
		if( !strncasecmp (data, ACT_TEXT[i], len)
		&&  is_atok_char(data[len]) )
			return ((REPLY_FNC)(ACT_FUNC[i]))(&data[len+1], mysql);
	}

	return ((REPLY_FNC)(ACT_FUNC[HELP]))(data, mysql);
}

/******************************************************************************/

#ifdef USE_MYSQL
#include <mysql/mysql.h>
int dbQueryMode = FALSE;
char *act_query (char const *data, void *mysql) {
	if(!strncmp("quit",data,strlen("quit"))) {
		dbQueryMode = FALSE;
		return strdup("dbexec end\n");
	}
	
	if(!data || strlen(data)<3) {
		if(dbQueryMode)
			return strdup("\n");
		else {
			dbQueryMode = TRUE;		
			return strdup("dbexec start\n");
		}
	}
	dbQueryMode = TRUE;		

	char *ret = NULL;
	if (!mysql_real_query(mysql, data, strlen(data))) { 
		MYSQL_RES *res = mysql_store_result(mysql);
		if(res) {			
			MYSQL_ROW row;
			unsigned long *lun, len = 0;
			int i, n = mysql_num_fields(res);
			ret = strdup("\n");
			while ((row = mysql_fetch_row(res))) {
				lun = mysql_fetch_lengths(res);
				for(i = 0; i < n; i++) 
					len += lun[i] + 4;
				if((ret = (char *)realloc(ret, len))) 
					for(i = 0; i < n; i++) 
						sprintf(ret, "%s%s%s", ret, i?"\t":"", row[i]);
				else
					break;
				strcat(ret, "\n");
			} 
			mysql_free_result(res);
		}
	} else {
		char tmp[1024];
		if(snprintf(tmp, 1024, "Query error: %s\n", mysql_error(mysql)) > 0)
			ret = strdup(tmp);
	}
	
	if(ret)
		return ret;
	return NULL;
}
#endif
