/***************************************************************************
 *            model.c
 *
 *  Thu Jun 16 15:14:24 2005
 *  Copyright  2005  Roberto A. Foglietta <roberto.foglietta@sad.it>
 *
 *  RAF, 30-06-2005: modified
 *
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <string.h>
#include "rafdebug.h"
#include "../include/iofile.h"
#include "../include/csv.h"

#define MAX_T 1024

enum {	FALSE = 0, TRUE };

static int log_dec(long n) {
	int a, b = 1;
	for(a = 0, b = 1; n > b; b*=10)
			a++;
	return a;
}

int main(int argc, char **argv) {
	DEBUG(OFF);
	long lun; 
	int j, i, n, nn, e;
	unsigned char *p[MAX_T], *buffer;
	csv_Split_t *row[MAX_T], *testa;

	if(argc < 3) {
		fprintf(stderr,	"\n"  
						"\n (C) 2005, Roberto A. Foglietta <roberto.foglietta@sad.it>"
						"\n 	Released under GPL v2 terms as part of Eltron P310c card print utility"
						"\n"
						"\n USAGE: %s filename.csv model.mod"
						"\n"
						"\nThe filename.csv is a file in which first row identify each colonum with a name."
						"\nFurther rows contains data which have to be substituted in model.mod, each row"
						"\ngenerate a new model.mod.N file, where N is the number of row (1...N)."
						"\n"	
						"\nThe model.mod contains the model in which #attributes# will be substituted with"
						"\nthe N-row value of the colonum identified by name contained between # symbols."
						"\n\n\n", basename(argv[0])
		);
		return 1;
	}
	
	if(!(buffer = read_file_to_buffer (argv[1], "rt", &lun))) {
		fprintf(stderr,"FATAL: read_file_to_buffer(\"%s\",...) return NULL buffer\n", argv[1]);
		return -1;
	}

	for(p[i=0] = strtok(buffer, "\n"); 
		i < MAX_T && p[i]; 
		p[++i] = strtok(NULL, "\n")) 
		PRINTF("p[%d]=%s", i, p[i]);
	n = i;
	i = 0; 
	while(p[i] && i < n) {
		if((e = csv_SplitNew((const char *)p[i], &row[i]))) {
			fprintf(stderr,"FATAL: csv_SplitNew(p[%d/%d], ...) return error code %d\n", i, n, e);
			return -2;
		}
		PRINTF("n(%d)=%d has %d fields", i, n, row[i]->num);
		i++;
	}
	PRINTF("\n");
	free(buffer);

	if(!(buffer = read_file_to_buffer (argv[2], "rt", &lun))) {
		fprintf(stderr,"FATAL: read_file_to_buffer(\"%s\",...) return NULL buffer\n", argv[2]);
		return -3;
	}

	testa = row[0];
	for(i = 1; i < n; i++) {
		char *buf = strdup(buffer);
		if(!buf) {
			fprintf(stderr,"FATAL: strdup(buffer) return NULL pointer\n");
			return -4;		
		}
		PRINTF("\n'%s'\n", buf);
		
		nn = 0;
		for(j = 0; j < testa->num; j++) {
			if(!strlen(row[i]->field[j])) 
				nn++;
		}
		if(nn >= testa->num) {
			fprintf(stderr,"FATAL: All fields are void in %d row\n", i);
			return -5;
		}
		
		for(j = 0; j < testa->num; j++) {
			PRINTF("strlen(testa->field[%d] = '%s') = %d ", j, row[i]->field[j], strlen(row[i]->field[j]));
			
			char *t, *p, needle[1024];
			snprintf(needle, 1024, "#%s#", testa->field[j]);
			if(!(p = strstr(buf, needle)))
				continue;
			
			if(!strlen(row[i]->field[j])) {
				char *pa = p;
				while(*pa != '\n' && pa > buf)  
					pa--;
				char *pb = strchr(p, '\n');
				if(pa && pb)
					bcopy(pb,pa,strlen(pb)+1);
			} else {			
				if(!(t = malloc(strlen(buf)+strlen(row[i]->field[j])+16))) {
					fprintf(stderr,"FATAL: malloc(buf+field) return NULL pointer\n");
					return -6;		
				}
				sprintf(t,buf);
				sprintf(&t[(int)(p-buf)], "%s%s", row[i]->field[j], (char *)(p+strlen(needle)));
				//free(buf);
				buf = t;
			}
			PRINTF("nn: %d/%d", nn, testa->num);
			j--;	//try again
		}
		
		int k = strlen(buf)-1;
		while(k > 0 && (buf[k] == '\n' || buf[k] == '\r'))
			buf[k--] = '\0';
	
		PRINTF("\n'%s'\n%d-------------------------------------------------------------\n",buf,i);
		//free(buf);
	
		char fmt[16], tmp[MAX_T];
		snprintf(fmt, 16, "%%s.%%0%dd", log_dec(n));
		PRINTF("fmt = %s\nargv[2]: '%s'\n", fmt, argv[2]);
		snprintf(tmp, MAX_T, fmt, basename(argv[2]), i);
		PRINTF("argv[2]: '%s'\n basename: '%s'\n tmp = '%s'\n", argv[2], basename(argv[2]), tmp);
		FILE *fp = fopen(tmp, "wt");
		if(!fp) {
			fprintf(stderr,"FATAL: fopen(\"%s\", ...) return NULL pointer\n", tmp);
			return -7;
		}
		fprintf(fp,buf);
		fclose(fp);	
		printf("%s\n", tmp);

	}
	return 0;
}
