/*
  This file is part of the Linux driver for the Eltron p310c plastic card printer
	Copyright (C) 2002 Flemming Frandsen <ff-p310c@partyticket.net>
	Based on initial work by Stian Skjelstad <stian@nixia.no>
	Materials (printer+cards+ribbons) provided by http://www.psi.no/
	
	Copyright (C) 2005, Roberto A. Foglietta <roberto.foglietta@sad.it> 
	Copyright (C) 2005, Roberto A. Foglietta <me@roberto.foglietta.name>
		
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.
*/

#ifndef IMAGES_H
#define IMAGES_H

#define YELLOW  0
#define MAGENTA 1
#define CYAN    2
#define BLACK   3

#define false 0
#define true  1 

typedef unsigned int Pixel; // A pixel in ABGR format

typedef struct {
    int width, height;
    Pixel *imagedata;
} Image;


// The size of the card (no it's probably not a good idea to hardcode it:):
#define HEIGHT 1008
#define WIDTH   624

typedef struct {
    int width, height;
    unsigned char *color[4];
} CMYKrImage;

// Copy an image onto another larger one at an offset.
void copyImg(Image src, Image dst, int x0, int y0);
Image makeImage(int width, int height);
void destroyImage(Image img);

CMYKrImage rgb2cmykr(Image *color, Image *black);
CMYKrImage makeCMYKrImage(int width, int height);
void destroyCMYKrImage(CMYKrImage cmy);
int saveCMYKrImage(char *fn, CMYKrImage *img);
CMYKrImage *loadCMYKrImage(char *fn);
char *colorrect(CMYKrImage img, int x, int y, int width, int height, int color);
char *monochrome(CMYKrImage img, int x, int y, int width, int height);

// This sets a pixel in a packed pixel bgra image:
#define setpixel(p,w,x,y,r,g,b,a) p[(x)+(y)*(w)] = ((a) << 24) | ((r)<<16) | ((g)<<8) | (b)

#endif
