/*
  This file is part of the Linux driver for the Eltron p310c plastic card printer
    Copyright (C) 2002 Flemming Frandsen <ff-p310c@partyticket.net>
    Copyright (C) 2002 Christian Laursen <xi@borderworlds.dk>
    Based on initial work by Stian Skjelstad <stian@nixia.no>
    Materials (printer+cards+ribbons) provided by http://www.psi.no/

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/
#include "pngloader.h"
#include <stdio.h>
#include <stdlib.h>
#include <png.h>

inline
unsigned long _trans(unsigned long z, int i) {
	static unsigned char cache[256], first=1;
	if(first) {
		first = 0;
		memset(cache, 0, 256);
		fprintf(stderr,"Gamma color correction level %d factor %1.2f\n", i, (float)_trans(128,i)/128);
	}
	if(i && z) {
		if(cache[z]) {
			z = cache[z];
		} else {
			unsigned long old, k=z;
			for(old=k; i>0; i-=2) { k++; k*=k; k>>=8; }
			if(i) { k+=old; k>>=1; }
			k++; if(k>0xff) k=0xff;
			cache[z] = k;
			z = k;
		}
	}
	return z;
}

#define trans(z) _trans(z,g)
//#define alpha(z) (row_pointers[y][z])		ALPHAs MAKE MIXED BEFORE GAMMA FILTER, NOT SUP. YET
#define alpha(z) (0)
#define pixel(z) (trans(row_pointers[y][z]))
#define bwpix(z) (((row_pointers[y][z/8]>>(7-(z%8)))&1)*0xffffff)

#define loadpixels(func) do {				\
  if(img.width == w && img.height == h) {		\
    for (y = 0; y < img.height; ++y) 			\
      for (x = 0; x < img.width; ++x) 			\
         img_data[y*img.width + x] = func;		\
  } else {						\
    for (y = 0; y < img.height; ++y) 			\
      for (x = 0; x < img.width; ++x) 			\
         img_data[(h-x-1)*w+y] = func;			\
    img.height = h;					\
    img.width  = w;					\
  }							\
} while(0)

Image png_load(char *filename, int w, int h, int r, int g)
{
  Image img;
  png_structp png_ptr = NULL;
  png_infop info_ptr, end_info;
  png_bytep *row_pointers;

  int x, y, color_type, is_png, nbit;
  unsigned int *img_data = NULL;
  unsigned char header[10];
  FILE *fp = NULL;
 
  if (!(fp = fopen(filename, "rb"))) {
    fprintf(stderr,"png_load: %s\n",filename);
    perror("fopen failed");
    exit(1);
  }

  /* Check the header */
  fread(header, 1, 8, fp);
  is_png = !png_sig_cmp(header, 0, 8);
  if (!is_png) {
    fprintf(stderr, "%s is not a png file\n", filename);
    exit(1);
  }

  /* Initialise the png library */
  png_ptr = png_create_read_struct (PNG_LIBPNG_VER_STRING, 
		(png_voidp)0, (png_error_ptr)0, (png_error_ptr)0);
  if (!png_ptr) {
    fprintf(stderr, "Couldn't allocate png_struct\n");
    exit(1);
  }

  info_ptr = png_create_info_struct(png_ptr);
  if (!info_ptr) {
    png_destroy_read_struct(&png_ptr, (png_infopp)NULL, (png_infopp)NULL);
    fprintf(stderr, "Couldn't allocate info_struct\n");
    exit(1);
  }

  end_info = png_create_info_struct(png_ptr);
  if (!end_info) {
    png_destroy_read_struct(&png_ptr, &info_ptr, (png_infopp)NULL);
    fprintf(stderr, "Couldn't allocate end info_struct\n");
    exit(1);
  }

  /* Initialise I/O */

  png_init_io(png_ptr, fp);
  png_set_sig_bytes(png_ptr, 8);

  /* Read the image */

  png_read_png(png_ptr, info_ptr, 0, NULL);
  row_pointers = png_get_rows(png_ptr, info_ptr);

  /* Image dimension */

  fprintf(stderr, "Loading image: %s ...\n",filename);
  img.width = png_get_image_width(png_ptr,info_ptr);
  img.height = png_get_image_height(png_ptr,info_ptr);
  if (img.width == h && img.height == w) {
    if(r) {
      fprintf(stderr, "Warning: degree 270 rotation will be applied\n");
    } else {
      fprintf(stderr, "Error: width=%d and height=%d are required\n",w,h);
      exit(1);
    }
  } else
  if (img.width == w && img.height == h) {
	;
  } else {
    fprintf(stderr, "Error: dimension %d x %d are required\n",w,h);
    exit(1);
  }

  /* Image color depth */

  nbit = png_get_bit_depth(png_ptr,info_ptr);
  if (nbit != 8 && nbit != 1) {
    fprintf(stderr, "Error: only 8 or 1 bit depth are supported\n");
    exit(1);
  }

  /* Loading Image */

  img_data = (unsigned int *)malloc(img.width * img.height * sizeof(int));
  color_type = png_get_color_type(png_ptr,info_ptr);
  switch(color_type) {
	case PNG_COLOR_TYPE_RGB:
		fprintf(stderr,"Image type is true color RGB\n");
		loadpixels(	(pixel(x*3)  <<16) +
				(pixel(x*3+1)<<8)  +
				(pixel(x*3+2))      );
		break;
	case PNG_COLOR_TYPE_RGB_ALPHA:
		fprintf(stderr,"Image type is true color RGB with alpha channel\n");
		loadpixels(	(alpha(x*4+3)<<24) +
				(pixel(x*4)  <<16) +
				(pixel(x*4+1)<<8)  +
				(pixel(x*4+2))      );
		break;
	case PNG_COLOR_TYPE_GRAY:
		fprintf(stderr,"Image type is 8 bits level gray\n");
		loadpixels(	(pixel(x)<<16) +
				(pixel(x)<<8)  +
				(pixel(x))      );
		break;
	case PNG_COLOR_TYPE_GRAY_ALPHA:
		fprintf(stderr,"Image type is 8 bits level gray with alpha channel\n");
		loadpixels(	(alpha(x*2+1)<<24) +
				(pixel(x*2)  <<16) +
				(pixel(x*2)  <<8)  +
				(pixel(x*2))        );
		break;
	case PNG_COLOR_TYPE_PALETTE:
		fprintf(stderr,"Image type is 1 bits level black and white\n");
		loadpixels(	bwpix(x) );
		break;
	default:
		fprintf(stderr, "Error: image uses palette or other unsupported crap\n");
		exit(1);
  }

  img.imagedata = img_data;
  png_destroy_read_struct(&png_ptr, &info_ptr, &end_info);

  return img;
}
