/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * COPYING NOTES
 *
 * main.c -- application core 
 *
 * Copyright (C) 2005, Roberto A. Foglietta <me@roberto.foglietta.name>
 * Copyright (C) 2005, SAD Trasporti Locali Bolzano s.p.a.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * REVISION NOTES:
 * Released 07-04-2005 by Roberto A. Foglietta
 * Modified 01-08-2005 by Roberto A. Foglietta
 */


#define VERSION __DATE__ __TIME__

#include <stdio.h>
#include <unistd.h>
//#include <string.h>
//#include <stdlib.h>
#include <syslog.h>
#include <stdarg.h>


#include "raflogdebug.h"
#include "replyfunc.h"
#include "mysignal.h"
#include "mainargs.h"
#include "server.h"
#include "iofile.h"

#ifdef USE_MYSQL
# include <mysql/mysql.h>
static MYSQL mysql;
#endif

int 
main (int argc, char **argv) {
	DEBUG(OFF);

	int ret = setup_args(argc, argv, VERSION);
	if(ret)
		return ret;
	
	INITLOG(basename(argv[0]), LOG_LPR, loglev);
	LOGINFO("Setting logmask upto level %d...", loglev);
	
	
	if(hostname)
		LOGINFO("HostName: %s", hostname);
#ifdef USE_MYSQL	
	if(dbport)
		LOGINFO("dbPort  : %d", dbport);
	if(dbname) 
		LOGINFO("DataBase: %s", dbname);
	if(username || password)
		LOGINFO("UserName: %s strlen(password)=%d", username, strlen(password));
	
	mysql_init(&mysql);
	if(!mysql_real_connect(&mysql, hostname, username, password, dbname, dbport, NULL, 0)) {
		LOGERRO("Connection to %s failed by %s@%s:%d", dbname, username, hostname, dbport);
		LOGERRO("Connection error was: '%s'", mysql_error(&mysql));
		return -3;
	}
	LOGINFO("Connected sucessfully to %s as %s@%s:%d", dbname, username, hostname, dbport);
#endif
	
	/********** LOOP PRINCIPALE **************************************************/
	
	signal (SIGINT, catch_int);
	LOGINFO("Looping in listening commands from socket...");
	do { 
#ifdef USE_MYSQL
		if(!my_socket_server(NULL, skport, reply_func, NULL, &mysql))
#else	
		if(!my_socket_server(hostname, skport, reply_func, NULL, NULL))
#endif
			break;
		else
			usleep(POLLING_PAUSE);
	} while(!CTRL_C_FLAG);
	LOGINFO("Listening loop ended\n");
	
	closelog();

	return 0;
}
