/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * COPYING NOTES
 *
 * replyfunc.c -- replay function bodies
 *
 * Copyright (C) 2002, Roberto A. Foglietta <robang@libero.it>
 * Copyright (C) 2002, GEA S.r.l. Generale Elettronica Automotive
 *
 * Copyright (C) 2004, Roberto A. Foglietta <me@roberto.foglietta.name>
 * Copyright (C) 2004, Tecno Elettrica Brianza snc di Riva P. & G.  
 *
 * Copyright (C) 2005, Roberto A. Foglietta <me@roberto.foglietta.name>
 * Copyright (C) 2005, SAD Trasporti Locali Bolzano s.p.a.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * REVISION NOTES:
 * released 21-10-2002 by Roberto A. Foglietta
 * modified 08-04-2005 by Roberto A. Foglietta
 * modified 12-04-2005 by Roberto A. Foglietta
 * modified 24-06-2005 by Roberto A. Foglietta
 * modified 02-08-2005 by Roberto A. Foglietta
 */


#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include "raflogdebug.h"
#include "replyfunc.h"

#ifdef USE_MYSQL
# include <mysql/mysql.h>
#endif

#define MAXLEN 2560
int NOUSE_TIMEOUT_s = 120;


/********* VERY GENERAL MACROS ************************************************/

//To force error if ; are not present at the end of the macro
#define done while(0)	
#define AUTOSIZE -1
#define STR(x)  #x
#define STR2(x) STR(x)
#define null 0


#define first_time_only(CMD) do{	\
	static int _first = TRUE;	 	\
	if(_first) {				 	\
		CMD; 				 		\
		_first = FALSE;				\
	}								\
}done

//TODO: sbagliato concettualmente
#define xmemsize(pstr, n) (			\
	((n) >= 0) ? 					\
		(n)*sizeof((pstr))			\
	: 								\
		sizeof((pstr))				\
)

#define rmvcrlf(string) do{											\
	int _i = xstrlen((string))-1;									\
	while(_i > 0 && ((string)[_i] == '\n' || (string)[_i] == '\r'))	\
		(string)[_i--] = '\0';										\
}done

//Safe substitute of snprintf 
#define xstrput(str, n, x...) (						\
	((str)) ? 										\
		snprintf((str), xmemsize((str),(n))-1, x)+1	\
	: 												\
		0 											\
)

#define nullsafe(pnt, CMD) ( 	\
	((pnt)) ?					\
		(CMD)					\
	: 							\
		null					\
)

//Safe substitute of memset
#define xmemset(pnt, i, lun) nullsafe((pnt), memset((pnt), (i), (lun)))

//Safe substitute of strlen
#define xstrlen(str) nullsafe((str), strlen((str)))

//Safe substitute of xstrchr
#define xstrchr(pnt, str) nullsafe((pnt), strchr((pnt), (str)))

//Safe substitute of xstrrchr
#define xstrrchr(pnt, str) nullsafe((pnt), strrchr((pnt), (str)))

//Safe substitute of strdup
#define xstrdup(str) nullsafe((str), strdup((str)))

//Safe substitute of malloc
#define xmalloc(x,n) (						\
	(n > 0) ?								\
		(x*)malloc((size_t)(n)*sizeof(x))	\
	:										\
		null								\
)

//Safe substitute of free
#define xfree(x) do{ 							\
	if((x) &&  sizeof((x)) == sizeof(void *)) { \
		free((x)); 								\
		setptonull((x));						\
	}											\
}done

#define setptox(p,x) do{ 	\
	if((p)) 			 	\
		*(p) = (x); 		\
}done

#define setptonull(p) do{	 		\
	if((p))							\
		bzero(&(p),sizeof(void *));	\
}done


/********* GENERAL MACROS *****************************************************/

#define FATAL(testo) "FATAL, %s\n\tin %s line %d", testo, __FILE__, __LINE__

#define answerput(p, stato, x...)  do{						\
	char b_str[MAXLEN];										\
	if(xstrput((b_str), AUTOSIZE, x) > 0) {					\
		(p) = answer(stato, b_str);							\
	} else {												\
		(p) = xstrdup((stato));								\
	}														\
}done

#define ritorna(stato, x...) do{							\
	char *b_p = NULL;										\
	answerput((b_p), (stato), x);							\
	return (b_p);											\
}done

enum { AOK, NAK, STP, ERR };

#define rispondi(level, x...) do{			\
	switch((level)) {						\
	case AOK:								\
		ritorna(_ACOK, x);					\
	case NAK:								\
		ritorna(_FAIL, x);					\
	case STP:								\
		ritorna(_STOP, x);					\
	case ERR: {								\
			char b_strn[MAXLEN];			\
			xstrput(b_strn, AUTOSIZE, x);	\
			ritorna(_FAIL, FATAL(b_strn));	\
		}									\
	}										\
	return NULL;							\
}done


/********* GENERAL FUNCTIONS **************************************************/

static
char *answer(const char *status, const char *string) {
	
	DEBUG(OFF);

	int     n = xstrlen(string);
	int     k = xstrlen(status);
	char *ret = NULL;
	
	PRINTF("n = %d, k = %d", n, k);

	if(n > 0) {	
		if(k > 0) {
			ret = xmalloc(char, n+k+16);
			PRINTF("nk) sizeof(ret) = '%d'", sizeof(ret));
			xstrput(ret, n+k+16, "%s: %s", status, string);
			PRINTF("nk) ret = '%s'", ret);
		} else {											
			ret = xstrdup(string);
			PRINTF("n ) ret = '%s'", ret);		
		}
	} else {
		if(!k)
			return NULL;
		ret = xstrdup(status);
		PRINTF(" k) ret = '%s'", ret);		
	}
	rmvcrlf(ret);
	
	LOGINFO("answer status: '%s'", status);
	//LOGDBUG(ret);

	return ret;
}

static 
int is_atok_char(int c) {
	static int atok_chars[] = {' ', '\n', '\r', ';',  '(', ')', EOF};

	int i = 0;
	while(atok_chars[i] != EOF)
		if(c == atok_chars[i++])
			return TRUE;
	
	return FALSE;
}

static
char *strduptrim(const char *str) {
	DEBUG(OFF);
	char *new, *p;

	if(!str)
		return NULL;
	
	while(is_atok_char(*str))
		str++;
	PRINTF("str: '%s'\n", str);
	
	new = strdup(str);
	if(!new)
		return NULL;
	PRINTF("new: '%s'\n", new);
	
	if((p = strchr(new, '\n'))) {
		PRINTF("p: '%s'\n", p);
		while(is_atok_char(*p) && p != new) {
			*p = '\0'; 
			p--;
			PRINTF("p: '%s'\n", p);
		}
	}
	PRINTF("new: '%s'\n", new);
	
	return new;
}

static
int what_action(const char *data)
{
	DEBUG(OFF);
	int i, len;
	if(is_atok_char(data[0]))
		return NUL_ACT;

	for (i = 0; i < N_ACT_FNC; i++) {
		len = strlen(ACT_TEXT[i]);
		PRINTF("i=%d, len=%d, data='%s' act_text='%s'\n", i, len, data, ACT_TEXT[i]);
		if( !strncasecmp (data, ACT_TEXT[i], len)
		&&  is_atok_char(data[len]) )
			return i;
	}

	return INV_ACT;
}

static 
int sncattsv(char *str, long slen, char **line, long nline) 
{
	long i, j = slen - strlen(str);

	if(!str || !line)
		return -1;

	for (i = 0; i < nline && j > 0; i++) {
		if(--j)
			strcat(str, "\t");
		if((j -= strlen(line[i])) > 0)
			strcat(str, line[i]);
		if(--j)
			strcat(str, "\n");
	}

	return !j;
}

static 
int filewrite(char *testo, char *nome) {
	FILE *fp = fopen(nome, "wt");
	if(!fp) return -1;
		
	int ret = 0;
	if(testo != NULL) 
		ret = fwrite(testo, strlen(testo), 1, fp);
	
	fclose(fp);
	return ret;
}

#include "iofile.h"
#include "csv.h"
#define MAX_T 1024

static
void **csvrows_from_buffer(char *buffer, int *nrows) 
{
	DEBUG(OFF);
	
	static csv_Split_t *row[MAX_T+1];						//TODO: da creare in modo dinamico
	xmemset(row, 0, xmemsize(row, AUTOSIZE));

	setptox(nrows,0);	
	if(!buffer)
		return NULL;
	
	int i = 0; 
	char *p = strtok(buffer, "\n");
	while(p) {
		PRINTF(".");
		if(csv_SplitNew((const char *)p, &row[i])) {
			row[i] = NULL;
			break;
		}
		
		PRINTF("row n.%d has %d fields", i, row[i]->num);
	
		i++;
		if(i >= MAX_T) {
			row[MAX_T] = NULL;
			i = -1;
			break;			
		}
		
		p = strtok(NULL, "\n");
	}

	setptox(nrows, i);
	PRINTF("nrows = 0x%lx and *nrows = %d", (long)nrows, (nrows)?(*nrows):i);

	return (void **)row;
}



/********* ANSWER DECLARATIONS ************************************************/

char *		ACT_HELP[N_ACT_FNC] = {	
	"hello: returns a hello from hostname",	
#ifdef USE_MYSQL
	"query SQL-query: returns query answer",   
#endif
	"help [(command)]: return list of commands or command help", 
	"bye: close the connection", 
	"timeout [(seconds)]: set/get timeout after which connection expire",
	"setcard (N, \"field1\", ..., \"fieldM\"): prepare a card and index it as n.N",
	"setmagn (N, \"track1\", ..., \"track3\"): prepare magnetic for card indexed as n.N",
//	"prtcrds (N[, ..., M]): print all cards indexed as n.N, ..., M\n\n",
	"print (N): print n.N copies of the card using csvline as info",
	"getheader: read and return the csvline header",
	"setheader [(csvhead)]: set the csvline header or a its subset",
	"setlayout (namedir): set the layout of printing card",
	"getlayout: lists all the available layouts",
	"sendcmd (command): where command could be RESET, EJECT or VERSION"
};

char *		ACT_TEXT[N_ACT_FNC] = {	
	"hello",	
#ifdef USE_MYSQL
	"query",   
#endif
	"help", 
	"bye", 
	"timeout",
	"setcard",
	"setmagn",
//	"prtcrds",
	"print",
	"getheader",
	"setheader",
	"setlayout",
	"getlayout",
	"sendcmd"
};
									
 REPLY_FNC	ACT_FUNC[N_ACT_FNC] = {	act_hello,	
#ifdef USE_MYSQL 					
	act_query, 				
#endif 
	act_help, 
	act_bye, 
	act_timeout,
	act_setcard,
	act_setmagn,
//	act_prtcrds,
	act_print,
	act_getheader,
	act_setheader,
	act_setlayout,
	act_getlayout,
	act_sendcmd
};


/********* COMMON ANSWER FUNCTIONS ********************************************/

char *act_hello(const char *data, void *mysql)
{
	char name[MAXLEN];
	int n = sprintf(name, "Hello from ");
	gethostname(&name[n], MAXLEN-n-1);
	rispondi(AOK, name);
}

char *act_bye(const char *data, void *mysql)
{
	rispondi(STP, "");
}

char *act_help(const char *data, void *mysql)
{
	int i = what_action(data);
	switch(i) {
	case INV_ACT:
	case NUL_ACT:
		break;
	default:
		rispondi(AOK, ACT_HELP[i]);
		break;
	}	

	char str[MAXLEN];
	snprintf(str, MAXLEN-1, "Admitted command list follow...\n");
	if(sncattsv(str, MAXLEN-1, ACT_TEXT, N_ACT_FNC)) {
		rispondi(NAK, FATAL("str[MAXLEN] is not enought long"));
	}
	rispondi(AOK, str);
}
//#undef MAXLEN


char *reply_func (char const *data, void *mysql)
{
	DEBUG(OFF);

#ifdef USE_MYSQL
	if(dbQueryMode)
		return ((REPLY_FNC)(ACT_FUNC[QUERY]))(data, mysql);
#endif

	char *ret = NULL;
	int len, i = what_action(data);
	
	switch(i) {
	case NUL_ACT:
		ret = strdup("");
		break;
	case INV_ACT:
		answerput(ret, _FAIL, "unknown command, try to use 'help'");
		break;
	default:
		LOGINFO("received command: '%s'", ACT_TEXT[i]);
		len = strlen(ACT_TEXT[i])+1;
		ret = ((REPLY_FNC)(ACT_FUNC[i]))(&data[len], mysql);
		break;
	}

	return ret;
}

/************************ EXTENDED ANSWER MACROS ******************************/

#define strnput_or_die(param, pstr, x...) do{						\
	int __n = xstrput((pstr), AUTOSIZE, x);							\
	if(__n >= xmemsize((pstr), AUTOSIZE)) {							\
		xfree((param));												\
		rispondi(ERR, "buffer " STR(pstr) " is not enough long");	\
	}																\
}done	

#define strdup_or_die(param, dest, pstr) do{				\
	if(!(pstr)) {											\
		xfree((param));										\
		rispondi(ERR, "duplication of NULL is not fair");	\
	}														\
	char *__p = xstrdup((pstr));							\
	if(__p == NULL) {										\
		xfree((param));										\
		rispondi(ERR, "strdup memory allocation failed");	\
	}														\
	xfree((dest));											\
	(dest) = __p;											\
}done


#define get_param_or_die(param, data) do{			\
 	(param) = strduptrim((data));					\
	if(!(param))									\
		rispondi(ERR, "memory allocation failure");	\
}done

#define check_param_or_die(param, data) do{						\
	get_param_or_die((param), (data));							\
	if(!(param[0]))												\
		rispondi(NAK, "at least one parameter ar required");	\
}done	

#define check_nextparam_or_die(param, next, data) do{	\
	if(data) 											\
		check_param_or_die((param),(data));				\
	if(param) {											\
		(next) = strchr(param, ',');					\
		if(!(next))										\
			rispondi(NAK, "more parameters expected");	\
		*(next) = '\0'; 								\
		while(*(++next) == ' ');						\
	} else {											\
		rispondi(ERR, "param or data expected");		\
	}													\
}done

#define check_layout_or_die(param) do{				\
	if(Cur.layout && Cur.layout[0]) 				\
	;else {											\
		xfree((param));								\
		rispondi(NAK, "current layout is not set");	\
	}												\
}done

#define check_header_or_die(param) do{					\
	check_layout_or_die((param));						\
	if(Cur.header && Cur.header[0] && Cur.csv_head) 	\
	;else {			 									\
		act_getheader(NULL, NULL);						\
		if(Cur.header && Cur.header[0] && Cur.csv_head)	\
		;else {											\
			xfree((param));								\
			rispondi(NAK, "header not found, sorry");	\
		}												\
	}													\
}done

#define check_card_or_die(param) do{			\
	check_header_or_die((param));				\
	if(!Cur.card) { 							\
		xfree(param);							\
		rispondi(NAK, "card not found, sorry");	\
	}											\
}done

#define check_header_numfields_or_die(param, str) do{								\
	check_header_or_die((param));													\
	int __n = 1; 																	\
	char *__p = (str);																\
	while((__p = xstrchr(__p, ','))) {												\
		__n++;																		\
		__p++;																		\
	}																				\
	if(__n != Cur.csv_head->num-1) {												\
		xfree((param));																\
		rispondi(NAK, "header (%d) and card (%d) have different number of fields", 	\
														Cur.csv_head->num-1, __n);	\
	}																				\
}done


/******************* EXTENDED ANSWER DATA STRUCT ******************************/

static struct print {
	short ready;			//FALSE

	char *layout;			//1NULL	<-------set

	char *header;			//2NULL <---+---set
	csv_Split_t *csv_head;	//3NULL	<---'

	char *card;				//4NULL <-------set

	char *magnet;			//5NULL

	short intprt;

} Cur = {FALSE, NULL, NULL, NULL, NULL, NULL, TRUE};

#undef  M
#define M(x)	cur_ ## x

#define setCurCard_or_die(param, buf) do{			\
	xfree(Cur.card);								\
	char *M(_p) = (buf);							\
	if(M(_p))										\
		strdup_or_die((param), Cur.card, M(_p));	\
}done

#define setCurHeader(head, buf) do{		\
	if(Cur.csv_head) {					\
		csv_SplitFree(&Cur.csv_head);	\
		Cur.csv_head = NULL;			\
	}									\
	Cur.csv_head = (head);				\
										\
	xfree(Cur.header);					\
	if((buf))							\
		setptox(&(Cur.header), (buf));	\
}done


#define setCurInitPrinter(value) do{	\
	Cur.intprt = (value);				\
}done

#define setCurLayout_or_die(param, buf) do{									\
	xfree(Cur.layout);														\
	char a_str[MAXLEN];														\
	strnput_or_die((param), a_str, "%s/%s/%s", SHARE_PATH, (buf), HEADER);	\
	FILE *a_fp = check_n_openfile(a_str, "rt");								\
	if(a_fp) { fclose(a_fp); } else {										\
		xfree((param));														\
		rispondi(NAK, "layout not found %s", a_str);						\
	}																		\
	Cur.layout = (buf);														\
	setptonull((buf));														\
}done




//#define MAXCARDS 50

//#define initprint(Cur) memset((void *)(&Cur), 0, sizeof(struct print))
	
#define TMP_DIRNAME  	"/tmp/eltron/"
#define CSV_FILENAME 	TMP_DIRNAME "print.csv"
#define CMD_FILENAME 	TMP_DIRNAME "print.command"
#define LAY_FILENAME 	TMP_DIRNAME "print.layout"
#define PRT_FILENAME	TMP_DIRNAME "print.start"
#define RES_FILENAME 	TMP_DIRNAME "print.result"
#define RETURN_STATUS_OK 	"return status: 0"
#define SHARE_PATH 		"/usr/local/share/eltron/"
#define MODELLO 		"modello.mod"
#define HEADER 			"header.csv"
#define SHELL 			"/bin/bash"

//#define filewrite(txt,fp) fwrite((txt), strlen((txt)), 1, (fp));



/*******************************************************************************
 * TODO:
 * Questo parametro deve poter essere impostato per ogni socket
 * e deve essere resettato ad ogni nuova chiusura di socket!!! 
 */

#include "server.h"
char *act_timeout  (const char *data, void *mysql) {
	char *param;
	get_param_or_die(param, data);

	int n = atoi(param);
	xfree(param);

	if(n > 0) {
		NOUSE_TIMEOUT_s = n;
		rispondi(AOK, "timeout was set at %d seconds", NOUSE_TIMEOUT_s);
	}
	rispondi(AOK, "current timeout is %d seconds", NOUSE_TIMEOUT_s);
}
/******************************************************************************/

#include <errno.h>

#define remove_or_die(param, filename) do{									\
	if(remove((filename)) < 0 && errno != ENOENT) {							\
		xfree((param));														\
		rispondi(ERR, "cannot remove %s, errno: %d", (filename), errno);	\
	}																		\
}done	

#define write_or_die(param, testo, filename) do{							\
	if(filewrite((testo), (filename)) < 0) {								\
		xfree((param));														\
		rispondi(ERR, "cannot write %s, errno: %d", (filename), errno);		\
	}																		\
}done



char *act_setcard  (const char *data, void *mysql) {

	DEBUG(OFF);
	
	char *p, *param = NULL;
	check_nextparam_or_die(param, p, data);

	if(atoi(param) != 1) {
		xfree(param);
		rispondi(NAK, "invalid line number, only N=1 is allowed");
	}

	check_header_numfields_or_die(param, p);
	remove_or_die(param, PRT_FILENAME);
	setCurCard_or_die(param, (p));

	PRINTF("\n%s\n1,%s\n", Cur.header, p);

	xfree(param);
	rispondi(AOK, "card n.1 has %d fields", Cur.csv_head->num-1);
}


char *act_setmagn  (const char *data, void *mysql) {

	DEBUG(OFF);	

	char *p, *pp, *param = NULL;
	check_header_or_die(param);
	check_nextparam_or_die(param, p, data);

	if(atoi(param) != 1) {
		xfree(param);
		rispondi(NAK, "invalid line number, only N=1 is allowed");
	}

	PRINTF("p = '%s'", p);
	
	char str[MAXLEN];
	pp = strtok(p, ",");
	int i = 0, n = 0, k = 0; 

	//questo ciclo dovrebbe finire con sncattsv
	while(pp) {
		if(++n > 3) {
			xfree(param);
			rispondi(NAK, "too many fields found, only 3 are allowed\n\n");
		}
		
		PRINTF("%d) pp = '%s'", n, pp);
	
		char *a;
		if((a = strchr(pp, '"')))
			pp = (++a);
		if((a = strrchr(pp, '"')))
			*a = '\0';
		if(strlen(pp)) {
			i += snprintf(&str[i],MAXLEN-i," -e%d \"%s\" ", n, pp);
			k++;
		}
		
		PRINTF("%d) pp = '%s'\n", n, pp);
	
		pp = strtok(NULL, ",");
	}
	xfree(param);

	if(i >= MAXLEN)
		rispondi(ERR, "str[MAXLEN] is not enought long");
	
	if(n < 3) 		
		rispondi(NAK, "too few fields found, just 3 are nedded");
		
	strdup_or_die(param, Cur.magnet, str);
	
	rispondi(AOK, "card n.1 has %d/3 not empty fields", k);
}


char *act_print  (const char *data, void *mysql) {

	char *param = NULL;
	check_card_or_die(param);
	check_param_or_die(param, data);
	
	int n = atoi(param);
	if(n < 1 || n > 50) {
		xfree(param);	
		rispondi(NAK, "%d is an invalid number of copies", n);
	} 
	xfree(param);

	char str[MAXLEN];	

/***** START WRITING ON FILES ******/

	strnput_or_die(param, str, "%s\n1,%s\n", Cur.header, Cur.card);
	write_or_die(param, str, CSV_FILENAME);

	strnput_or_die(param, str, "%s/%s/%s", SHARE_PATH, Cur.layout, MODELLO);
	write_or_die(param, str, LAY_FILENAME);

	strnput_or_die(param, str, "%s -n %d -N %s", Cur.magnet?Cur.magnet:"", n, Cur.intprt?"on":"off");
	write_or_die(param, str, PRT_FILENAME);

	setCurInitPrinter(FALSE);

/*****  END  WRITING ON FILES ******/

	int trip = 0;
	FILE *fp = NULL;
	do {
		sleep(1);
		trip++;
	
		fp = check_n_openfile(CSV_FILENAME, "rt");
		if(fp) 
			fclose(fp);
	} while(fp && trip < 60+10*n);								//DA MIGLIORARE
		
	long lun;
	char *buf = read_file_to_buffer(RES_FILENAME, "rt", &lun);
	if(!buf || !lun)  
		rispondi(ERR, "failed reading print response file");
	snprintf(str, MAXLEN, "printing n.%d cards\n%s\n\n", n, buf);
	
	char *out = strstr(buf, RETURN_STATUS_OK);
	free(buf);
	
	rispondi((out)?AOK:NAK, str);
}

static
char *strlower(char *str) {
	char *p = str;
	while(p && *p) {
		if(*p >= 'A' && *p <= 'Z')
			*p += 'a'-'A';
		*p++;
	}
	return str;
}

char *act_sendcmd  (const char *data, void *mysql) {

	char *param = NULL;
	check_param_or_die(param, data);
	strlower(param);

	char str[MAXLEN];
	if(strstr(param,"eject")) {
		sprintf(str, "--eject");
	} else 
	if(strstr(param,"reset")) { 
		sprintf(str, "--reset");
	} else 
	if(strstr(param,"version")) { 
		sprintf(str, "--version");
	} else {
		xfree(param);	
		rispondi(NAK, "unknown commad '%s'", param);
	}
	
/***** START WRITING ON FILES ******/

	write_or_die(param, str, CMD_FILENAME);

/*****  END  WRITING ON FILES ******/

	int trip = 0;
	FILE *fp = NULL;
	do {
		sleep(1);
		trip++;
	
		fp = check_n_openfile(CMD_FILENAME, "rt");
		if(fp) 
			fclose(fp);
	} while(fp && trip < 60);
		
	long lun;
	char *buf = read_file_to_buffer(RES_FILENAME, "rt", &lun);
	if(!buf || !lun)  
		rispondi(ERR, "failed reading command response file");
	
	snprintf(str, MAXLEN, "executing command '%s'\n%s\n\n", param, buf);
	char *out = strstr(buf, RETURN_STATUS_OK);
	free(buf);
	
	rispondi((out)?AOK:NAK, str);
}


char *act_setheader(const char *data, void *mysql) {
	rispondi(ERR, "not implemented yet");
}


char *act_getheader (const char *data, void *mysql) {
	
	DEBUG(OFF);

	char *param = NULL;
	check_layout_or_die(param);
	setCurCard_or_die(param, null);
	setCurHeader(null, null);
	
	char str[MAXLEN];
	strnput_or_die(param, str, "%s/%s/%s", SHARE_PATH, Cur.layout, HEADER);

	PRINTF("reading %s", str);

	char *buffer = read_file_to_buffer (str, "rt", NULL);
	if(!buffer)
		rispondi(ERR, "error reading csv file %s", str);
	
	PRINTF("buffer = '%s'", buffer);
	
	int nrows;
	csv_Split_t **row = (csv_Split_t **)csvrows_from_buffer(buffer, &nrows);	
	if(row && row[0] && nrows == 1) 
	;else {
		xfree(buffer);
		rispondi(ERR, "unknown error parsing on %s\n"
			"\treturn %d rows (row:0x%lx, row[0]:0x%lx)", str, nrows, (long)row, (long)row[0]);
	}
	csv_Split_t *testa = row[0];
	setCurHeader(testa,buffer);

	int j;
	char *p[MAX_T];							//TODO: confusione fra numero righe e colonne
	for(j = 0; j < testa->num; j++) {
		PRINTF("j = %d", j);
		if(j > MAX_T)
			rispondi(ERR, "p[MAX_T] is not enought long");
		p[j] = testa->field[j];
	}

	strnput_or_die(param, str, "%s, %d\n", Cur.layout, testa->num-1);
	if(sncattsv(str, MAXLEN, &p[1], testa->num-1))
		rispondi(ERR, "str[MAXLEN] is not enought long");
	rispondi(AOK, str);
}


char *act_setlayout  (const char *data, void *mysql) {

	char *param = NULL;
	check_param_or_die(param, data);

	setCurHeader(null, null);
	setCurCard_or_die(param, null);
	setCurLayout_or_die(param, param);
	setCurInitPrinter(TRUE);

	remove_or_die(param, PRT_FILENAME);
	remove_or_die(param, CSV_FILENAME);

	rispondi(AOK, Cur.layout);
}

#include <sys/types.h>
#include <dirent.h>

char *act_getlayout  (const char *data, void *mysql) {

	struct dirent **namelist;
    int n, tot, nfn, err = FALSE;;
	long i, len;
	char *str;

	nfn = scandir(SHARE_PATH, &namelist, 0, alphasort);
    if (nfn < 0)
		rispondi(ERR, "failed reading layouts directory %s", SHARE_PATH);

	len = 0;
	tot = 0;
	n = nfn;
	while(--n) {
		if(namelist[n]->d_type == DT_DIR && namelist[n]->d_name[0] != '.') {
			len += strlen(namelist[n]->d_name)+4;
			tot += 1; 
		}
    }
	if((str = (char *)malloc(len+16)) == NULL) 
		rispondi(ERR, "failed allocating memory");
	
	n = nfn;
	i = sprintf(str, "%d\n", tot);
    while(--n) {
		if(namelist[n]->d_type == DT_DIR && namelist[n]->d_name[0] != '.') {
			if(!err && i+strlen(namelist[n]->d_name)+4 < len) {
				i += sprintf(&str[i], "\t%s\n", namelist[n]->d_name);
			} else {
				err = TRUE;
			}
		}
		free(namelist[n]);
    }
    free(namelist);
	
	rispondi((err)?ERR:AOK, str);
}



/******************************************************************************/

#ifdef USE_MYSQL
#include <mysql/mysql.h>
int dbQueryMode = FALSE;
char *act_query (char const *data, void *mysql) {
	if(!strncmp("quit",data,strlen("quit"))) {
		dbQueryMode = FALSE;
		return strdup("dbexec end\n");
	}
	
	if(!data || strlen(data)<3) {
		if(dbQueryMode)
			return strdup("\n");
		else {
			dbQueryMode = TRUE;		
			return strdup("dbexec start\n");
		}
	}
	dbQueryMode = TRUE;		

	char *ret = NULL;
	if (!mysql_real_query(mysql, data, strlen(data))) { 
		MYSQL_RES *res = mysql_store_result(mysql);
		if(res) {			
			MYSQL_ROW row;
			unsigned long *lun, len = 0;
			int i, n = mysql_num_fields(res);
			ret = strdup("\n");
			while ((row = mysql_fetch_row(res))) {
				lun = mysql_fetch_lengths(res);
				for(i = 0; i < n; i++) 
					len += lun[i] + 4;
				if((ret = (char *)realloc(ret, len))) 
					for(i = 0; i < n; i++) 
						sprintf(ret, "%s%s%s", ret, i?"\t":"", row[i]);
				else
					break;
				strcat(ret, "\n");
			} 
			mysql_free_result(res);
		}
	} else {
		char tmp[1024];
		if(snprintf(tmp, 1024, "Query error: %s\n", mysql_error(mysql)) > 0)
			ret = strdup(tmp);
	}
	
	if(ret)
		return ret;
	return NULL;
}
#endif
